/********************************************************************\
 * gnc-helpers.c -- gnucash g-wrap helper functions                 *
 * Copyright (C) 2000 Linas Vepstas                                 *
 *                                                                  *
 * This program is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU General Public License as   *
 * published by the Free Software Foundation; either version 2 of   *
 * the License, or (at your option) any later version.              *
 *                                                                  *
 * This program is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public License*
 * along with this program; if not, contact:                        *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 *                                                                  *
\********************************************************************/

#include "gnc-helpers.h"

#include <g-wrap.h>

#include "gnc.h"
#include "Group.h"
#include "GNCId.h"
#include "messages.h"


SCM
gnc_guid2scm(GUID guid) {
  const char *string;

  string = guid_to_string(&guid);

  return gh_str02scm((char *)string);
}

GUID
gnc_scm2guid(SCM guid_scm) {
  char string[GUID_ENCODING_LENGTH + 1];
  GUID guid;

  gh_get_substr(guid_scm, string, 0, GUID_ENCODING_LENGTH);
  string[GUID_ENCODING_LENGTH] = '\0';

  string_to_guid(string, &guid);

  return guid;
}

int
gnc_guid_p(SCM guid_scm) {
  char string[GUID_ENCODING_LENGTH + 1];
  GUID guid;

  if (!gh_string_p(guid_scm))
    return (0 == 1);

  gh_get_substr(guid_scm, string, 0, GUID_ENCODING_LENGTH);
  string[GUID_ENCODING_LENGTH] = '\0';

  return string_to_guid(string, &guid);
}

GUID
gnc_account_get_guid(Account *account) {
  return *xaccAccountGetGUID(account);
}

Account *
gnc_account_lookup(GUID guid) {
  return xaccAccountLookup(&guid);
}

SCM
gnc_timespec2timepair(Timespec t) {
  SCM secs;
  SCM nsecs;

  secs = gh_longlong2scm(t.tv_sec);
  nsecs = gh_long2scm(t.tv_nsec);
  return(gh_cons(secs, nsecs));
}

Timespec
gnc_timepair2timespec(SCM x) {
  Timespec result;

  result.tv_sec = gh_scm2longlong(gh_car(x));
  result.tv_nsec = gh_scm2long(gh_cdr(x));
  return(result);
}

int
gnc_timepair_p(SCM x) {
  return(gh_pair_p(x) && gh_exact_p(gh_car(x)) && gh_exact_p(gh_cdr(x)));
}

Timespec
gnc_transaction_get_date_posted(Transaction *t) {
  Timespec result;
  xaccTransGetDateTS(t, &result);
  return(result);
}

Timespec
gnc_transaction_get_date_entered(Transaction *t) {
  Timespec result;
  xaccTransGetDateEnteredTS(t, &result);
  return(result);
}

Timespec
gnc_split_get_date_reconciled(Split *s) {
  Timespec result;
  xaccSplitGetDateReconciledTS(s, &result);
  return(result);
}

void
gnc_transaction_set_date_posted(Transaction *t, const Timespec d) {
  xaccTransSetDateTS(t, &d);
}

void
gnc_transaction_set_date_entered(Transaction *t, const Timespec d) {
  xaccTransSetDateEnteredTS(t, &d);
}

void
gnc_transaction_set_date(Transaction *t, Timespec ts)
{
  xaccTransSetDateTS(t, &ts);
}

char *
gnc_gettext_helper(const char *string)
{
  return _(string);
}

static int
gnc_scm_traversal_adapter(Transaction *t, void *data)
{
  POINTER_TOKEN pt;
  SCM scm_trans, result;
  SCM thunk = *((SCM *) data);

  pt = make_POINTER_TOKEN("Transaction*", t);
  scm_trans = POINTER_TOKEN_to_SCM(pt);

  result = gh_call1(thunk, scm_trans);

  if (result == SCM_BOOL_F)
    return(GNC_F);
  else
    return(GNC_T);
}

gncBoolean
gnc_scmGroupStagedTransactionTraversal(AccountGroup *grp,
                                       unsigned int new_marker,
                                       SCM thunk)
{
  return xaccGroupStagedTransactionTraversal(grp, new_marker,
                                             gnc_scm_traversal_adapter,
                                             &thunk);
}

gncBoolean
gnc_scmAccountStagedTransactionTraversal(Account *a,
                                         unsigned int new_marker,
                                         SCM thunk) 
{
  return xaccAccountStagedTransactionTraversal(a, new_marker,
					       gnc_scm_traversal_adapter,
                                               &thunk);
}
